
# q-expansion of newform 2304.2.a.r, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2304
weight = 2
poly_data = [-2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [0, 1], [0, -1], [-4, 0], [0, 2], [2, 0], [4, 0], [0, -2], [0, -1], [0, 3], [0, 0], [10, 0], [12, 0], [0, 2], [0, -1], [4, 0], [0, -4], [4, 0], [0, 2], [2, 0], [0, 3], [4, 0], [6, 0], [14, 0], [0, -5], [0, 1], [-12, 0], [0, -2], [-2, 0], [0, -3], [12, 0], [10, 0], [-4, 0], [0, 5], [0, 3], [0, 0], [4, 0], [0, -4], [0, -7], [12, 0], [0, -2], [0, -4], [6, 0], [0, 3], [0, 1], [-12, 0], [0, -7], [-12, 0], [0, 2], [6, 0], [0, 8], [6, 0], [12, 0], [30, 0], [0, 0], [0, 3], [0, -7], [0, 6], [-10, 0], [-4, 0], [0, 7], [4, 0], [0, -4], [14, 0], [0, 3], [-20, 0], [-22, 0], [-36, 0], [0, -4], [-18, 0], [0, 10], [0, 3], [0, -12], [28, 0], [0, 4], [0, -11], [0, -12], [-30, 0], [-10, 0], [-28, 0], [0, 6], [0, -10], [-34, 0], [0, -11], [-20, 0], [18, 0], [6, 0], [0, -3], [0, -7], [-28, 0], [0, 10], [0, -5], [20, 0], [4, 0], [0, 2], [0, -1], [-6, 0], [-20, 0], [0, 6], [20, 0], [0, -3], [4, 0], [-6, 0], [-4, 0], [-22, 0], [36, 0], [-34, 0], [0, -14], [34, 0], [0, 5], [0, -4], [38, 0], [-36, 0], [0, 3], [34, 0], [-44, 0], [0, 14], [0, -15], [-4, 0], [0, 16], [14, 0], [0, -7], [-20, 0], [-28, 0], [0, 7], [0, -6], [0, 6], [0, -9], [0, -6], [36, 0], [0, 8], [0, -17], [0, 10], [42, 0], [38, 0], [0, -1], [20, 0], [0, -7], [10, 0], [-20, 0], [0, 11], [0, 7], [-12, 0], [0, -18], [0, -2], [0, -8], [-6, 0], [44, 0], [0, -20], [0, 4], [46, 0], [-12, 0], [0, 0], [44, 0], [0, -8], [0, -1], [18, 0], [42, 0], [0, 5], [-36, 0], [42, 0], [0, 5], [12, 0], [2, 0], [0, 0], [0, -9], [0, -12], [-2, 0], [0, -15], [-60, 0], [0, -8], [0, 8], [10, 0], [0, 3], [22, 0], [-20, 0], [0, 13], [0, 5], [0, 0], [0, -21], [36, 0], [0, -6], [6, 0], [0, 4], [0, 9], [0, -6], [-28, 0], [-18, 0], [0, 16], [2, 0], [-28, 0], [52, 0], [0, 13], [-68, 0], [-26, 0], [26, 0], [0, 10], [14, 0], [0, 4], [0, 1], [0, 21], [0, -10], [26, 0], [12, 0], [0, 11], [0, 5], [-44, 0], [6, 0], [28, 0], [22, 0], [0, 1], [0, 21], [20, 0], [0, -18], [38, 0], [0, 15], [18, 0], [0, -12], [0, 25], [0, -18], [0, -23], [34, 0], [0, -11], [-12, 0], [0, 16], [6, 0], [0, 18], [0, 9], [20, 0], [0, -18], [-44, 0], [0, 1], [-26, 0], [-20, 0], [0, 2], [-34, 0], [0, -19], [28, 0], [0, -18], [-44, 0], [-20, 0], [0, 7], [0, -24], [30, 0], [0, -26], [0, 3], [28, 0], [28, 0], [0, 20], [0, 6], [46, 0], [0, 14], [-54, 0], [0, 3], [76, 0], [0, -18], [-52, 0], [0, -17], [-42, 0], [0, 17], [12, 0], [0, 6], [0, 8], [-62, 0], [68, 0], [0, -7], [54, 0], [-36, 0], [0, 5], [0, -12], [-44, 0], [-38, 0], [0, 19], [50, 0], [0, 9], [12, 0], [0, 28], [50, 0], [-36, 0], [0, -14], [0, -3], [0, -30], [0, -12], [60, 0], [0, 12], [-26, 0], [0, 29], [0, -9], [-82, 0], [0, -23], [-4, 0], [-58, 0], [-44, 0], [0, -16], [0, 21], [0, 9], [0, -21], [44, 0], [36, 0], [-34, 0], [0, -11], [0, 3], [-4, 0], [60, 0], [-10, 0], [60, 0], [0, 18], [0, 14], [0, 12], [0, 4], [0, -21], [-30, 0], [4, 0], [0, 4], [42, 0], [-4, 0], [0, -2], [-38, 0], [-18, 0], [4, 0], [-50, 0], [0, -25], [0, -13], [-86, 0], [62, 0], [52, 0], [-36, 0], [0, 0], [0, -7], [0, 14], [0, 25], [0, 9], [60, 0], [92, 0], [-68, 0], [0, -8], [-66, 0], [-30, 0], [0, 17], [0, 24], [-54, 0], [0, 11], [0, 33], [0, 7], [-12, 0], [0, 18], [-20, 0], [0, -26], [0, 31], [20, 0], [6, 0], [0, -5], [0, -11], [0, 24], [6, 0], [0, -16], [20, 0], [-14, 0], [0, -4], [0, -8], [54, 0], [0, 12], [-36, 0], [-28, 0], [-46, 0], [0, 31], [0, 27], [-10, 0], [-12, 0], [-84, 0], [0, -24], [0, -17], [0, -17], [0, 10], [84, 0], [0, 4], [-62, 0], [18, 0], [-2, 0], [0, -31], [-38, 0], [0, 27], [-66, 0], [-60, 0], [0, -20], [0, -9], [0, -10], [-4, 0], [0, -26], [2, 0], [0, 1], [4, 0], [-44, 0], [0, 32], [54, 0], [0, 9], [-6, 0], [-68, 0], [0, 9], [0, 20], [-94, 0], [0, -15], [-6, 0], [0, -9], [0, -1], [0, 6], [66, 0], [20, 0], [-60, 0], [-6, 0], [0, 5], [12, 0], [-28, 0], [-62, 0], [0, -25], [0, 4], [0, -21], [66, 0], [0, 30], [0, -19], [0, 0], [0, 20], [-84, 0], [30, 0], [0, -9], [-60, 0], [70, 0], [-68, 0], [0, 14]]
